% explicit finite difference solver for thermal pressurization equations
% constant friction coefficient and slip velocity,
% slip-on-plane or Gaussian shear zone
% written for SCEC TPV105 code verification problem
% Jan 15, 2011 Eric M. Dunham (edunham@stanford.edu)

refine = 1; % mesh refinement factor (set to 1,2,4,8,... to test convergence)

% material properties and physical parameters

w = 0; % half-width of shear zone [mm], set w=0 for slip-on-plane
alpha_th = 1; % thermal diffusivity [mm^2/s]
alpha_hy = 1e2; % hydraulic diffusivity [mm^2/s]
Lambda = 0.1; % undrained pore pressure change / temperature change [MPa/K]
rho_c = 2.7e-3; % volumetric heat capacity [GPa/K]

% friction, normal stress, and slip velocity
f = 0.2; % friction coefficient
sigma = 120; % normal stress [MPa]
V = 1; % slip velocity [m/s]

% spatial and temporal discretization

dz = 1/refine; % unrefined grid spacing [mm]
nz = 100*refine+1; % number of grid points

dt = 0.25*dz^2/max(alpha_th,alpha_hy); % unrefined time step [s], ~0.5*dz^2/max(alpha_th,alpha_hy)
nt = round(12/dt)+1; % number of time steps
t = [0:nt-1]*dt; % time vector

% coordinate vector

z = [0:nz-1]*dz;
  
% temperature and pressure fields

T = zeros(1,nz);
p = zeros(1,nz);
  
% temperature and pressure history on fault

p0 = zeros(1,nt);
T0 = zeros(1,nt);

I = 2:(nz+1); % interior computational points (without ghost points)

% store fields on fault

T0(1) = T(1);
p0(1) = p(1);

for n=1:nt-1

  % calculate heat production
  
  tau = f*(sigma-p(1));
  tauV = tau*V;
  if w==0 % slip on plane
      omega = zeros(1,nz);
      omega(1) = tauV/dz;
  else % Gaussian shear zone
      omega = tauV/(w*sqrt(2*pi))*exp(-0.5*(z/w).^2);
  end
  
  % use finite difference approximation dT/dz=[T(+dz)-T(-dz)]/(2*dz)
  % to set value in 'ghost' grid points outside computational domain
  % (this sets Neumann boundary conditions)
  
  Tm = T(2)   ; % set dT/dz=0 at z=0 (symmetry b.c.)
  Tp = T(nz-1); % set dT/dz=0 at z=zmax (artificial b.c.)
  
  pm = p(2   ); % set dp/dz=0 at z=0 (symmetry b.c.)
  pp = p(nz-1); % set dp/dz=0 at z=zmax (artificial b.c.)
  
  % pad with ghost cells
  
  To = [Tm T Tp];
  po = [pm p pp];
    
  % update fields
  
  % diffusion
  dT = alpha_th*(To(I+1)-2*To(I)+To(I-1))/dz^2+omega/rho_c;
  dp = alpha_hy*(po(I+1)-2*po(I)+po(I-1))/dz^2+Lambda*dT;

  % undrained, adiabatic
  %dT = omega/rho_c;
  %dp = Lambda*dT;
  
  T = T+dt*dT;
  p = p+dt*dp;

  % store fields on fault
  
  T0(n+1) = T(1);
  p0(n+1) = p(1);

end
  
tau = f*(sigma-p0); % shear strength history

%return

% plots

% analytical solution (Rice, JGR, 2006) for slip-on-a-plane
% with constant f and V

Lstar = 4/f^2*(rho_c/Lambda)^2*(sqrt(alpha_hy)+sqrt(alpha_th))^2/V;
delta = V*t;
tau_exact = f*(sigma-p0(1))*exp(delta/Lstar).*erfc(sqrt(delta/Lstar));

plot(delta,tau_exact,'b',delta,tau,'r')
%plot(delta,tau_exact,'b-o',delta,tau,'r.')
xlim([0 max(t)])
xlabel('slip (m)')
ylabel('shear strength (MPa)')
legend('exact (w=0)','numerical')
if w==0
  L2err = norm(tau-tau_exact)/sqrt(nt);
  title(['dz = ' num2str(dz) ' mm, L2 (rms) error = ' num2str(L2err) ' MPa'])
else
  title(['dz = ' num2str(dz) ' mm'])
end

